@@ -1,4 +1,4 @@
-# Copyright (c) 2006-2017  Andrey Golovigin
+# Copyright (c) 2006-2021  Andrey Golovizin
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -26,44 +26,47 @@ Usage:
 >>> t = Text('this ', 'is a ', Tag('em', 'very'), Text(' rich', ' text'))
 >>> print(t.render_as('latex'))
 this is a \emph{very} rich text
->>> print(str(t))
+>>> print(six.text_type(t))
 this is a very rich text
 >>> t = t.capitalize().add_period()
 >>> print(t.render_as('latex'))
 This is a \emph{very} rich text.
->>> print(str(t))
+>>> print(six.text_type(t))
 This is a very rich text.
 >>> print(Symbol('ndash').render_as('latex'))
 --
 >>> t = Text('Some ', Tag('em', Text('nested ', Tag('tt', 'Text', Text(' objects')))), '.')
 >>> print(t.render_as('latex'))
 Some \emph{nested \texttt{Text objects}}.
->>> print(str(t))
+>>> print(six.text_type(t))
 Some nested Text objects.
 >>> t = t.upper()
 >>> print(t.render_as('latex'))
 SOME \emph{NESTED \texttt{TEXT OBJECTS}}.
->>> print(str(t))
+>>> print(six.text_type(t))
 SOME NESTED TEXT OBJECTS.
 
 >>> t = Text(', ').join(['one', 'two', Tag('em', 'three')])
 >>> print(t.render_as('latex'))
 one, two, \emph{three}
->>> print(str(t))
+>>> print(six.text_type(t))
 one, two, three
 >>> t = Text(Symbol('nbsp')).join(['one', 'two', Tag('em', 'three')])
 >>> print(t.render_as('latex'))
 one~two~\emph{three}
->>> print(str(t))
+>>> print(six.text_type(t))
 one<nbsp>two<nbsp>three
 """
+from __future__ import absolute_import, unicode_literals
 
-
-import warnings
 import itertools
+import warnings
 from abc import ABCMeta, abstractmethod
+
+import six
 from pybtex import textutils
-from pybtex.utils import deprecated, collect_iterable
+from pybtex.utils import collect_iterable, deprecated
+from pybtex import py3compat
 
 
 # workaround for doctests in Python 2/3
@@ -71,7 +74,7 @@ def str_repr(string):
     """
     >>> print(str_repr('test'))
     'test'
-    >>> print(str_repr('test'))
+    >>> print(str_repr(u'test'))
     'test'
     """
 
@@ -83,7 +86,7 @@ def str_repr(string):
 
 
 def ensure_text(value):
-    if isinstance(value, str):
+    if isinstance(value, six.string_types):
         return String(value)
     elif isinstance(value, BaseText):
         return value
@@ -92,7 +95,10 @@ def ensure_text(value):
         raise ValueError('parts must be strings or BaseText instances, not ' + bad_type)
 
 
-class BaseText(object, metaclass=ABCMeta):
+@py3compat.python_2_unicode_compatible
+class BaseText(object):
+    __metaclass__ = ABCMeta
+
     @abstractmethod
     def __str__(self):
         raise NotImplementedError
@@ -146,9 +152,9 @@ class BaseText(object, metaclass=ABCMeta):
         """Join a list using this text (like string.join)
 
         >>> letters = ['a', 'b', 'c']
-        >>> print(str(String('-').join(letters)))
+        >>> print(six.text_type(String('-').join(letters)))
         a-b-c
-        >>> print(str(String('-').join(iter(letters))))
+        >>> print(six.text_type(String('-').join(iter(letters))))
         a-b-c
         """
 
@@ -196,11 +202,11 @@ class BaseText(object, metaclass=ABCMeta):
         Add a period to the end of text, if the last character is not ".", "!" or "?".
 
         >>> text = Text("That's all, folks")
-        >>> print(str(text.add_period()))
+        >>> print(six.text_type(text.add_period()))
         That's all, folks.
 
         >>> text = Text("That's all, folks!")
-        >>> print(str(text.add_period()))
+        >>> print(six.text_type(text.add_period()))
         That's all, folks!
 
         """
@@ -300,6 +306,7 @@ class BaseText(object, metaclass=ABCMeta):
         return None, ()
 
 
+@py3compat.python_2_unicode_compatible
 class BaseMultipartText(BaseText):
     info = ()
 
@@ -336,7 +343,7 @@ class BaseMultipartText(BaseText):
         self.length = sum(len(part) for part in self.parts)
 
     def __str__(self):
-        return ''.join(str(part) for part in self.parts)
+        return ''.join(six.text_type(part) for part in self.parts)
 
     def __eq__(self, other):
         """
@@ -383,7 +390,7 @@ class BaseMultipartText(BaseText):
         False
 
         """
-        if not isinstance(item, str):
+        if not isinstance(item, six.string_types):
             raise TypeError(item)
         return not item or any(part.__contains__(item) for part in self.parts)
 
@@ -398,7 +405,7 @@ class BaseMultipartText(BaseText):
         Text(Tag('em', '!'))
         """
 
-        if isinstance(key, int):
+        if isinstance(key, six.integer_types):
             start = key
             end = None
         elif isinstance(key, slice):
@@ -621,7 +628,7 @@ class BaseMultipartText(BaseText):
 
     @deprecated('0.19', 'use __unicode__() instead')
     def plaintext(self):
-        return str(self)
+        return six.text_type(self)
 
     @deprecated('0.19')
     def enumerate(self):
@@ -682,6 +689,7 @@ class BaseMultipartText(BaseText):
         return self._create_similar(iter_map_with_condition())
 
 
+@py3compat.python_2_unicode_compatible
 class String(BaseText):
     """
     A :py:class:`String` is a wrapper for a plain Python string.
@@ -701,7 +709,7 @@ class String(BaseText):
         All arguments must be plain unicode strings.
         Arguments are concatenated together.
 
-        >>> print(str(String('November', ', ', 'December', '.')))
+        >>> print(six.text_type(String('November', ', ', 'December', '.')))
         November, December.
         """
 
@@ -711,7 +719,7 @@ class String(BaseText):
         return str_repr(self.value)
 
     def __str__(self):
-        return str(self.value)
+        return six.text_type(self.value)
 
     def __eq__(self, other):
         """
@@ -740,7 +748,7 @@ class String(BaseText):
         if sep is None:
             from .textutils import whitespace_re
             parts = whitespace_re.split(self.value)
-        elif isinstance(sep, str):
+        elif isinstance(sep, six.string_types):
             parts = self.value.split(sep)
         else:
             try:
@@ -781,7 +789,7 @@ class String(BaseText):
 
     @property
     def parts(self):
-        return [str(self)]
+        return [six.text_type(self)]
 
     def _typeinfo(self):
         return String, ()
@@ -830,19 +838,19 @@ class Tag(BaseMultipartText):
 
     def __check_name(self, name):
         depr_map = {}
-        depr_map['emph'] = 'em'
+        depr_map[u'emph'] = u'em'
         if name in depr_map:
-            msg = "The tag '%s' is deprecated" % name
-            msg += ", use '%s' instead." % depr_map[name]
+            msg = u"The tag '%s' is deprecated" % name
+            msg += u", use '%s' instead." % depr_map[name]
             warnings.warn(msg, DeprecationWarning, stacklevel=3)
             return depr_map[name]
         return name
 
     def __init__(self, name, *args):
-        if not isinstance(name, (str, Text)):
+        if not isinstance(name, (six.string_types, Text)):
             raise ValueError(
                 "name must be str or Text (got %s)" % name.__class__.__name__)
-        self.name = self.__check_name(str(name))
+        self.name = self.__check_name(six.text_type(name))
         self.info = self.name,
         super(Tag, self).__init__(*args)
 
@@ -859,7 +867,7 @@ class Tag(BaseMultipartText):
 
 
 class HRef(BaseMultipartText):
-    r"""
+    """
     A :py:class:`HRef` represends a hyperlink:
 
     >>> from pybtex.richtext import Tag
@@ -867,21 +875,21 @@ class HRef(BaseMultipartText):
     >>> print(href.render_as('html'))
     <a href="http://ctan.org/">CTAN</a>
     >>> print(href.render_as('latex'))
-    \href{http://ctan.org/}{CTAN}
+    \\href{http://ctan.org/}{CTAN}
 
     >>> href = HRef(String('http://ctan.org/'), String('http://ctan.org/'))
     >>> print(href.render_as('latex'))
-    \url{http://ctan.org/}
+    \\url{http://ctan.org/}
 
     :py:class:`HRef` supports the same methods as :py:class:`Text`.
 
     """
 
     def __init__(self, url, *args):
-        if not isinstance(url, (str, BaseText)):
+        if not isinstance(url, (six.string_types, BaseText)):
             raise ValueError(
                 "url must be str or Text (got %s)" % url.__class__.__name__)
-        self.url = str(url)
+        self.url = six.text_type(url)
         self.info = self.url,
         super(HRef, self).__init__(*args)
 
@@ -948,6 +956,7 @@ class Protected(BaseMultipartText):
         return backend.format_protected(text)
 
 
+@py3compat.python_2_unicode_compatible
 class Symbol(BaseText):
     """A special symbol. This class is rarely used and may be removed in
     future versions.
@@ -969,7 +978,7 @@ class Symbol(BaseText):
 
     def __str__(self):
         # XXX
-        return '<%s>' % self.name
+        return u'<%s>' % self.name
 
     def __eq__(self, other):
         return self.name == other.name
@@ -1009,4 +1018,3 @@ class Symbol(BaseText):
 
 
 nbsp = Symbol('nbsp')
-
